package org.wso2.carbon.registry.app;

import org.wso2.carbon.registry.core.Registry;
import org.wso2.carbon.registry.core.RegistryConstants;
import org.wso2.carbon.registry.core.session.UserRegistry;
import org.wso2.carbon.registry.core.jdbc.EmbeddedRegistry;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.apache.axis2.context.MessageContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.axiom.om.util.Base64;

import javax.servlet.http.HttpServletRequest;
import java.security.MessageDigest;

public class Utils {
    private static final Log log = LogFactory.getLog(Utils.class);

    private static EmbeddedRegistry embeddedRegistry;

    public static synchronized void setEmbeddedRegistry(EmbeddedRegistry embRegistry) {
        embeddedRegistry = embRegistry;
    }

    public static Registry getRegistry() throws RegistryException {

        MessageContext messageContext = MessageContext.getCurrentMessageContext();
        if (messageContext == null) {
            String msg = "Could not get the user's Registry session. Message context not found.";
            log.error(msg);
            throw new RegistryException(msg);
        }

        HttpServletRequest request =
                (HttpServletRequest) messageContext.getProperty("transport.http.servletRequest");

        Registry registry =
                (Registry) request.getSession().getAttribute(RegistryConstants.USER_REGISTRY);

        if (registry == null) {

            String msg = "User's Registry instance is not found. " +
                    "Creating a anonymous Registry instance for the user.";
            if (log.isDebugEnabled()) {
                log.debug(msg);
            }

            registry = getEmbeddedRegistry().getUserRegistry(RegistryConstants.ADMIN_USER,
                    RegistryConstants.ADMIN_PASSWORD);
            request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, registry);
        }

        return registry;
    }

    public static UserRegistry getRegistry(HttpServletRequest request) throws RegistryException {

        UserRegistry registry =
                (UserRegistry) request.getSession().getAttribute(RegistryConstants.USER_REGISTRY);
        if (registry == null) {
            String auth = request.getHeader("Authorization");
            if (auth != null && auth.length() > 0) {
                String[] aParts = auth.trim().split(" ");
                if (aParts.length == 2) {
                    String credentials = aParts[1];
                    String decodedCredentials = new String(Base64.decode(credentials));
                    String[] cParts = decodedCredentials.trim().split(":");
                    if (cParts.length == 2) {
                        String userName = cParts[0];
                        String password = cParts[1];

                        registry = getEmbeddedRegistry().getUserRegistry(userName, password);
                        request.getSession()
                                .setAttribute(RegistryConstants.USER_REGISTRY, registry);
                    }
                }
            }
        }
        if (registry == null) {

            String msg = "User's Registry instance is not found. " +
                    "Creating a anonymous Registry instance for the user.";
            if (log.isDebugEnabled()) {
                log.debug(msg);
            }

            registry = getEmbeddedRegistry().getUserRegistry();
            request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, registry);
        }

        return registry;
    }

    public static EmbeddedRegistry getEmbeddedRegistry() throws RegistryException {
        if (embeddedRegistry == null) {
            String msg = "Embedded Registry service is not available. Make sure that the " +
                    "required version of the Registry core component is properly intalled.";
            log.error(msg);
            throw new RegistryException(msg);
        }

        return embeddedRegistry;
    }

    private static char[] hexMap = {'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};

    public static String getUUID(String uri){
        char[] tempArray = new char[36];
        try{
            MessageDigest md5 = MessageDigest.getInstance("MD5");
            md5.update(uri.getBytes());
            byte[] password = md5.digest();
            int count = 0;
            for(int i=0;i<password.length;i++){

                if(count==8 || count==13 || count==18 || count==23){
                    tempArray[count++] = '-';
                }
                //if(count==8 || count==13 ||count==18||count==23){
                //    tempArray[i] = '-';
                //    continue;
                //}

                //System.out.println(Integer.toHexString(password[i]));

                //tempArray[i] = (char)(password[i] & 0x00000000F);
                //tempArray[count] = (char)((password[i] & 0x0000000F) >> 4);

                tempArray[count++] = hexMap[(password[i] & 0x000000F0) >> 4];
                tempArray[count++] = hexMap[(password[i] & 0x0000000F)];

            }
        }catch(Exception e){
            e.printStackTrace();
        }
        return (new String(tempArray));
    }
}
