/*
 *  Copyright (c) 2005-2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */
package org.wso2.carbon.registry.core.test.jdbc;

import org.wso2.carbon.registry.core.test.utils.BaseTestCase;
import org.wso2.carbon.registry.core.jdbc.EmbeddedRegistry;
import org.wso2.carbon.registry.core.jdbc.InMemoryEmbeddedRegistry;
import org.wso2.carbon.registry.core.jdbc.realm.RegistryRealm;
import org.wso2.carbon.registry.core.Registry;
import org.wso2.carbon.registry.core.RegistryConstants;
import org.wso2.carbon.registry.core.Resource;
import org.wso2.carbon.registry.core.Comment;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import java.io.*;

public class FileRestoreTest extends BaseTestCase {

    protected static EmbeddedRegistry embeddedRegistry = null;
    protected static Registry registry = null;
    protected static RegistryRealm realm = null;

    public void setUp() {
        super.setUp();
        if (embeddedRegistry != null) {
            return;
        }
        try {
            embeddedRegistry = new InMemoryEmbeddedRegistry();
            registry = embeddedRegistry.getUserRegistry(
                    RegistryConstants.ADMIN_USER, RegistryConstants.ADMIN_PASSWORD);
        } catch (RegistryException e) {
            fail("Failed to initialize the registry. Caused by: " + e.getMessage());
        }
    }

    public void testFileRestore() throws RegistryException, XMLStreamException {

        Resource r = registry.newResource();
        r.setProperty("key1", "value1");
        r.setProperty("key2", "value2");
        r.setContent("content 1");
        registry.put("/testDump", r);

        registry.addComment("/testDump", new Comment("comment1"));
        registry.addComment("/testDump", new Comment("comment2"));

        registry.applyTag("/testDump", "tag1");
        registry.applyTag("/testDump", "tag2");

        registry.rateResource("/testDump", 3);

        // doing the dump
        StringWriter writer = new StringWriter();
        registry.dump("/testDump", writer);
        Reader input = new StringReader(writer.toString());

        //create the parser
        XMLStreamReader parser = XMLInputFactory.newInstance().createXMLStreamReader(input);
        //create the builder
        StAXOMBuilder builder = new StAXOMBuilder(parser);

        // get the element to restore
        OMElement dumpElement = builder.getDocumentElement();

        OMElement contentElement = dumpElement.getFirstChildWithName(new QName("content"));
        contentElement.setText(""); // make the content empty.

        File tempFile = new File("tempFile_test.txt");
        try {
            BufferedWriter out = new BufferedWriter(new FileWriter(tempFile));
            out.write("new content");
            out.close();
        } catch(IOException e) {
            // just try to write something to file
            throw new RegistryException(e.getMessage());
        }

        // now set the filename to the content file attribute
        contentElement.addAttribute("filename", "tempFile_test.txt", null);
        dumpElement.addAttribute("uploadContentFromFiles", "true", null);

        Reader reader = new StringReader(dumpElement.toString());

        registry.restore("/testDump", reader);

        r = registry.get("/testDump");

        String content = new String((byte[])r.getContent());
        assertEquals("Content is not equal", content, "new content");

        // setting filepath an absolute path
        contentElement.addAttribute("filename", "/bin/version.txt", null);

        reader = new StringReader(dumpElement.toString());
        registry.restore("/testDump", reader);

        r = registry.get("/testDump");

        content = new String((byte[])r.getContent());
        assertEquals("Content is not equal", content, "${product.name} v${product.version}\n");
    }
}
