package org.wso2.carbon.ui;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.BundleContext;
import org.wso2.carbon.CarbonConstants;
import org.wso2.carbon.ui.deployment.beans.CarbonUIDefinitions;
import org.wso2.carbon.ui.deployment.beans.Menu;
import org.wso2.carbon.ui.internal.CarbonUIServiceComponent;
import org.wso2.carbon.utils.CarbonUtils;
import org.wso2.carbon.utils.NetworkUtils;
import org.wso2.carbon.utils.ServerConfiguration;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Locale;

/**
 * Utility class for Carbon UI
 */
public class CarbonUIUtil {
    private static Log log = LogFactory.getLog(CarbonUIUtil.class);
    private static BundleContext bundleContext = null;

    public static void setBundleContext(BundleContext context) {
        bundleContext = context;
    }

    public static BundleContext getBundleContext() {
        return bundleContext;
    }

    public static String getServerURL(ServerConfiguration serverConfig) {
        return getServerURL(serverConfig.getFirstProperty(CarbonConstants.SERVER_URL));
    }
    
    public static String getServerURL(ServletContext servletContext, HttpSession httpSession) {
        String url;
        Object obj = httpSession.getAttribute(CarbonConstants.SERVER_URL);
        if (obj != null && obj instanceof String) {
            //Server URL is present in the servlet session
            url = (String) obj;
        } else {
            url = (String) servletContext.getAttribute(CarbonConstants.SERVER_URL);
        }
        return getServerURL(url);
    }

    private static String getServerURL(String url) {
        if(url.indexOf("${carbon.https.port}") != -1){
            String httpsPort =
                    CarbonUtils.
                            getTransportPort(CarbonUIServiceComponent.getConfigurationContextService(),
                                             "https") + "";
            url = url.replace("${carbon.https.port}", httpsPort);
        }
        return url;
    }

   /**
     * Returns url to admin console.
     * eg: https://192.168.1.201:9443/wso2/carbon
     *
     * @param request The HTTPServletRequest
     * @return The URL of the Admin Console
     */
    public static String getAdminConsoleURL(HttpServletRequest request) {

        // Hostname
        String hostName = "localhost";
        try {
            hostName = NetworkUtils.getLocalHostname();
        } catch (Exception ignored) {
        }

        // HTTPS port
        int httpsPort =
                CarbonUtils.getTransportPort(CarbonUIServiceComponent.getConfigurationContextService(),
                                             "https");

        // Context
        String context = request.getContextPath();
        if ("/".equals(context)) {
            context = "";
        }

        return "https://" + hostName + ":" + httpsPort + context + "/carbon/";
    }

    /**
     * Returns url to admin console.
     *
     * @param context Webapp context root of the Carbon webapp
     * @return The URL of the Admin Console
     */
    public static String getAdminConsoleURL(String context){
        // Hostname
        String hostName = "localhost";
        try {
            hostName = NetworkUtils.getLocalHostname();
        } catch (Exception ignored) {
        }

        // HTTPS port
        int httpsPort =
                CarbonUtils.getTransportPort(CarbonUIServiceComponent.getConfigurationContextService(),
                                             "https");

        // Context
        if ("/".equals(context)) {
            context = "";
        }

        return "https://" + hostName + ":" + httpsPort + context + "/carbon/";
    }

    /**
     * Get a ServerConfiguration Property
     * @param propertyName  Name of the property
     * @return  the property
     */
    public static String getServerConfigurationProperty(String propertyName){
        try {
            ServerConfiguration serverConfig = CarbonUIServiceComponent.getServerConfiguration();
            return serverConfig.getFirstProperty(propertyName);
        } catch (Exception e) {
            String msg = "ServerConfiguration Service not available";
            log.error(msg, e);
        }

        return null;
    }

    public static boolean isContextRegistered(ServletConfig config, String context) {
        URL url;
        try {
            url = config.getServletContext().getResource(context);
        } catch (MalformedURLException e) {
            return false;
        }
        if (url == null) {
            return false;
        } else if (url.toString().indexOf(context) != -1) {
            return true;
        }
        return false;
    }

    /**
     * Returns internationalized string for supplied key.
     *
     * @param key        - key to look for
     * @param i18nBundle - resource bundle
     * @param language   - language
     * @return internationalized key
     *         value of key, if no value can be derived
     */
    public static String geti18nString(String key, String i18nBundle, String language) {
        Locale locale = new Locale(language);
        String text = geti18nString(key, i18nBundle, locale);
        return text;
    }

    /**
     * Returns internationalized string for supplied key.
     *
     * @param key        - key to look for
     * @param i18nBundle - resource bundle
     * @param locale     - locale
     * @return internationalized key
     *         value of key, if no value can be derived
     */
    public static String geti18nString(String key, String i18nBundle, Locale locale) {
        java.util.ResourceBundle resourceBundle = null;
        try {
            resourceBundle = java.util.ResourceBundle.getBundle(i18nBundle, locale);
        } catch (java.util.MissingResourceException e) {
            if (log.isDebugEnabled()) {
                log.debug("Cannot find resource bundle : " + i18nBundle
                          + " for locale : " + locale);
            }
        }

        String text = key;
        if (resourceBundle != null) {
            String tmp = null;
            try {
                tmp = resourceBundle.getString(key);
            } catch (java.util.MissingResourceException e) {
                //Missing key should not be a blocking factor for UI rendering
                if (log.isDebugEnabled()) {
                    log.debug("Cannot find resource for key :" + key);
                }
            }
            if (tmp != null) {
                text = tmp;
            }
        }
        return text;
    }

    /**
     * Removed menu item from current user's session.
     * Only current user's menu items are effected.
     *
     * @param menuId
     * @param request
     * @see CarbonUIDefinitions#removeMenuDefinition(String)
     */
    public static void removeMenuDefinition(String menuId, HttpServletRequest request) {
        //TODO : consider removing child menu items as well
        ArrayList<Menu> modifiedMenuDefs = new ArrayList<Menu>();
        Menu[] currentMenus = (Menu[]) request.getSession().getAttribute(MenuAdminClient.USER_MENU_ITEMS);
        boolean modified = false;
        if (currentMenus != null) {
            if (menuId != null && menuId.trim().length() > 0) {
                for (int a = 0; a < currentMenus.length; a++) {
                    Menu menu = currentMenus[a];
                    if (menu != null) {
                        if (!menuId.equals(menu.getId())) {
                            modifiedMenuDefs.add(menu);
                            modified = true;
                        } else {
                            if (log.isDebugEnabled()) {
                                log.debug("Removing menu item : " + menuId);
                            }
                        }
                    }
                }
                if (modified) {
                    Menu[] newMenuDefs = new Menu[modifiedMenuDefs.size()];
                    newMenuDefs = modifiedMenuDefs.toArray(newMenuDefs);
                    request.getSession().setAttribute(MenuAdminClient.USER_MENU_ITEMS, newMenuDefs);
				}
			}
		}
	}
    
    /**
	 * This will remove menu items which are no longer requires after user being signed in
	 * successfully. TODO: This method needs to be improved to fire an event to the corresponding UI
	 * component - so the component it self could handle this logic.
	 * 
	 * @param request
	 */
	public static void removeMenuDefinitionOnSignIn(HttpServletRequest request) {
		removeMenuDefinition("user_registration_menu", request);
		removeMenuDefinition("infocard_sign_in", request);
	}
}